from linebot.models import (
    MessageEvent, TextMessage, TextSendMessage,TemplateSendMessage,ImageSendMessage, StickerSendMessage, AudioSendMessage
)
from linebot.models.template import *
from linebot import (
    LineBotApi, WebhookHandler
)

from boto.s3.connection import S3Connection
from boto.s3.key import Key as S3Key


class lineevent():
  def __init__(self,event,s3info):
    self.event = event
    self.eventtype = event['type']
    self.timestamp = event['timestamp']
    self.userid = event['source']['userId']
    self.rtoken = event['replyToken']
    self.aws_access_key_id = s3info['AWS_ACCESS_KEY_ID']
    self.aws_secret_access_key = s3info['AWS_SECRET_ACCESS_KEY']
    self.imagebucket = s3info['imagebucket']
    self.audiobucket = s3info['audiobucket']
    if self.eventtype == 'message':
      messageevent = self.event['message']
      messagetype = messageevent['type']
      if messagetype == 'text':
        messageinput = messageevent['text']
        self.result = messageinput
      elif messagetype == 'image':
        message_content = line_bot_api.get_message_content(messageevent['id'])
        imfile = 'im_'+str(self.userid)+'_'+str(self.timestamp)+'.jpg'
        with open(imfile, 'wb') as fd:
          for chunk in message_content.iter_content():
            fd.write(chunk)
        #result = upload(imfile, public_id=imfile)
        result = self.upload_s3(imfile, '.jpg', self.imagebucket)
        print(result)
        self.result = result
      elif messagetype == 'audio':
        message_content = line_bot_api.get_message_content(messageevent['id'])
        imfile = 'au_'+str(self.userid)+'_'+str(self.timestamp)+'.wav'
        with open(imfile, 'wb') as fd:
          for chunk in message_content.iter_content():
            fd.write(chunk)
        result = self.upload_s3(imfile, '.wav', self.audiobucket)
        print(result)
        self.result = result

  def upload_s3(self, file, content_type, bucket_name):
    s3connection = S3Connection(self.aws_access_key_id, self.aws_secret_access_key)
    bucket = s3connection.get_bucket(bucket_name)
    obj = S3Key(bucket)
    obj.name = file
    obj.set_metadata('Content-Type', content_type)
    obj.set_contents_from_filename(file)
    obj.set_acl('public-read')
    return obj.generate_url(expires_in=0, query_auth=False)










import requests
import re

USER_AGENT = 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.181 Safari/537.36'
headers = { 'User-Agent': USER_AGENT }
 
def get_image_urls(query_key,nres=50,iter=50):
    """
        Get all image url from google image search
        Args:
            query_key: search term as of what is input to search box.
        Returns:
            (list): list of url for respective images.
 
    """
    urllist = []
    i = 0
    query_key = query_key.replace(' ','+')#replace space in query space with +
    query = query_key
    while (len(urllist)<nres)|(i>iter):
      print('complete: %.2f%%'%(len(urllist)/nres*100))
      tgt_url = 'https://www.google.co.th/search?q={}&tbm=isch&tbs=sbd:0'.format(query)#last part is the sort by relv
      r = requests.get(tgt_url, headers = headers)
      urllist = urllist + [n for n in re.findall('"(https[a-zA-Z0-9_./:-]+.(?:jpg|jpeg|png))",', r.text)]
      urllist = list(set(urllist))
      randomstr = getrandstring()
      query = query_key + randomstr
    print('completed')
    return urllist

import string
import numpy as np
def getrandstring():
  letters = string.ascii_letters + string.digits
  letters = list(letters)
  letters = ''.join(np.random.choice(letters,2))
  return letters


import pandas as pd
import glob
from pythainlp.tokenize import word_tokenize
import pickle
import numpy as np
import os
#import pkg_resources
#path = 'botnoiw2v_small.mod'
#modloc = pkg_resources.resource_filename(__name__,path)

#mod = pickle.load(open(modloc))
def sentencevector(senList,mod):
	vecList = []
	for i in range(len(senList)):
		sen = senList[i]
		print('encode: %i/%i'%(i+1,len(senList)))
		wList = word_tokenize(str(sen),engine='newmm')
		wvec = []
		for w in wList:
			try:
				wvec.append(mod[w])
			except:
				pass
		if len(wvec)==0:
			vec = np.zeros(50)
		else:
			vec = np.mean(wvec,0)
		vecList.append(vec)
	return vecList


import io
import PIL
import pickle
import requests
from sklearn.preprocessing import normalize
import os

def extract_bowtfidf(textList):
  '''extract bow with tfidf vectors'''
  from botnoi import getbow as gb
  modpath = os.path.join(os.path.dirname(gb.__file__),'botnoitfidf_v1.mod')
  mod = pickle.load(open(modpath,'rb'))
  feat = gb.sentencevector(textList,mod)
  feat = normalize(feat)
  return feat

def extract_w2vlight(textList):
  '''extract w2v vectors'''
  from botnoi import getw2v as gw
  modpath = os.path.join(os.path.dirname(gw.__file__),'botnoiw2v_small.mod')
  mod = pickle.load(open(modpath,'rb'))
  feat = gw.sentencevector(textList,mod)
  feat = normalize(feat)
  return 

def printhello():
  print('hello')






import pandas as pd
import glob
import pickle
import numpy as np
import os
#import pkg_resources
#path = 'botnoiw2v_small.mod'
#modloc = pkg_resources.resource_filename(__name__,path)

#mod = pickle.load(open(modloc))
def sentencevector(sentenceList,mod):
	return mod.transform(sentenceList)


from keras.applications.imagenet_utils import preprocess_input
from keras.preprocessing import image
from keras.applications.mobilenet import MobileNet
from os.path import join
from PIL import Image, ImageOps
import glob
import numpy as np
import urllib.request
from sklearn.preprocessing import normalize
import pandas as pd
import requests
import io
import PIL

model = MobileNet(input_shape=(224,224,3),alpha=1.0,depth_multiplier=1,dropout=1e-3,include_top=True,weights='imagenet',input_tensor=None,pooling=None,classes=1000)

def extract_feature(imgpath):
  if imgpath.find('http')!=-1:
      r = requests.get(imgpath, allow_redirects=True, timeout=10)
      image_bytes = io.BytesIO(r.content)
      img = PIL.Image.open(image_bytes)
  else:
  	  img = image.load_img(imgpath)
  img = ImageOps.fit(img, (224, 224), Image.ANTIALIAS)
  x = image.img_to_array(img)
  x = np.expand_dims(x, axis=0)
  x = preprocess_input(x)
  features = model.predict(x, batch_size=1,verbose=0)
  features = np.ndarray.flatten(features).astype('float64')
  feat = normalize([features])[0]
  return feat


import os
os.environ['TF_CPP_MIN_LOG_LEVEL']='3'
import argparse
import numpy as np
import sys
from keras.applications.resnet50 import ResNet50
from keras.applications.imagenet_utils import preprocess_input
from keras.preprocessing import image
from os.path import join
from PIL import Image, ImageOps
import glob
import numpy as np
import urllib.request
import requests
import io
import PIL
from sklearn.preprocessing import normalize

model = ResNet50(include_top=False, weights='imagenet', classes=1000)

def extract_feature(imgpath):
  if imgpath.find('http')!=-1:
      r = requests.get(imgpath, allow_redirects=True, timeout=10)
      image_bytes = io.BytesIO(r.content)
      img = PIL.Image.open(image_bytes)
  else:
  	  img = image.load_img(imgpath)
  img = ImageOps.fit(img, (224, 224), Image.ANTIALIAS)
  x = image.img_to_array(img)
  x = np.expand_dims(x, axis=0)
  x = preprocess_input(x)
  features = model.predict(x, batch_size=1,verbose=0)
  features = np.ndarray.flatten(features).astype('float64')
  feat = normalize([features])[0]
  return feat


import io
import PIL
import pickle
import requests
class image():
  def __init__(self,imgpath):
    self.imgpath = imgpath

  def getmobilenet(self):
    from botnoi import mobilenet as mn
    mfeat = mn.extract_feature(self.imgpath)
    self.mobilenet = mfeat
    return mfeat

  def getresnet50(self):
    from botnoi import resnet50 as rn
    rfeat = rn.extract_feature(self.imgpath)
    self.resnet50 = rfeat
    return rfeat

  def getimage(self):
    r = requests.get(self.imgpath, allow_redirects=True, timeout=10)
    image_bytes = io.BytesIO(r.content)
    img = PIL.Image.open(image_bytes)
    self.image = img
    return img

  def save(self,filename):
    pickle.dump(self,open(filename,'wb'))






class lineevent():
  def __init__(self,event):
    self.event = event
    self.eventtype = event['events'][0]['type']
    self.msgid  = event['events'][0]['message']['id']
    self.time = event['events'][0]['timestamp']
    self.token = event['events'][0]['replyToken']
    self.idpage = event['destination']
    self.userid = event['events'][0]['source']['userId']
  def extractmessage(self):
    self.message = event['events'][0]['message']['text']





